@@ -23,7 +23,7 @@
 
 #include "gck.h"
 
-#include "pkcs11.h"
+#include <p11-kit/pkcs11.h>
 #include "pkcs11i.h"
 
 #ifndef GCK_MOCK_H
