@@ -23,7 +23,6 @@
 
 #include "config.h"
 
-#include "gcr-base.h"
 #define DEBUG_FLAG GCR_DEBUG_TRUST
 #include "gcr-debug.h"
 #include "gcr-types.h"
@@ -115,22 +114,20 @@ trust_closure_free (gpointer data)
 	g_free (closure);
 }
 
-static GckAttributes*
-prepare_trust_attrs (GcrCertificate *certificate, CK_X_ASSERTION_TYPE type)
+static void
+prepare_trust_attrs (GcrCertificate *certificate,
+                     CK_X_ASSERTION_TYPE type,
+                     GckBuilder *builder)
 {
-	GckAttributes *attrs;
 	gconstpointer data;
 	gsize n_data;
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_X_TRUST_ASSERTION);
-	gck_attributes_add_ulong (attrs, CKA_X_ASSERTION_TYPE, type);
+	gck_builder_add_ulong (builder, CKA_CLASS, CKO_X_TRUST_ASSERTION);
+	gck_builder_add_ulong (builder, CKA_X_ASSERTION_TYPE, type);
 
 	data = gcr_certificate_get_der_data (certificate, &n_data);
-	g_return_val_if_fail (data, NULL);
-	gck_attributes_add_data (attrs, CKA_X_CERTIFICATE_VALUE, data, n_data);
-
-	return attrs;
+	g_return_if_fail (data);
+	gck_builder_add_data (builder, CKA_X_CERTIFICATE_VALUE, data, n_data);
 }
 
 /* ----------------------------------------------------------------------------------
@@ -140,15 +137,14 @@ prepare_trust_attrs (GcrCertificate *certificate, CK_X_ASSERTION_TYPE type)
 static GckAttributes *
 prepare_is_certificate_pinned (GcrCertificate *certificate, const gchar *purpose, const gchar *peer)
 {
-	GckAttributes *attrs;
+	GckBuilder builder = GCK_BUILDER_INIT;
 
-	attrs = prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE);
-	g_return_val_if_fail (attrs, NULL);
+	prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE, &builder);
 
-	gck_attributes_add_string (attrs, CKA_X_PURPOSE, purpose);
-	gck_attributes_add_string (attrs, CKA_X_PEER, peer);
+	gck_builder_add_string (&builder, CKA_X_PURPOSE, purpose);
+	gck_builder_add_string (&builder, CKA_X_PEER, peer);
 
-	return attrs;
+	return gck_attributes_ref_sink (gck_builder_end (&builder));
 }
 
 static gboolean
@@ -312,16 +308,15 @@ gcr_trust_is_certificate_pinned_finish (GAsyncResult *result, GError **error)
 static GckAttributes *
 prepare_add_pinned_certificate (GcrCertificate *certificate, const gchar *purpose, const gchar *peer)
 {
-	GckAttributes *attrs;
+	GckBuilder builder = GCK_BUILDER_INIT;
 
-	attrs = prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE);
-	g_return_val_if_fail (attrs, NULL);
+	prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE, &builder);
 
-	gck_attributes_add_string (attrs, CKA_X_PURPOSE, purpose);
-	gck_attributes_add_string (attrs, CKA_X_PEER, peer);
-	gck_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
+	gck_builder_add_string (&builder, CKA_X_PURPOSE, purpose);
+	gck_builder_add_string (&builder, CKA_X_PEER, peer);
+	gck_builder_add_boolean (&builder, CKA_TOKEN, TRUE);
 
-	return attrs;
+	return gck_attributes_ref_sink (gck_builder_end (&builder));
 }
 
 static gboolean
@@ -329,7 +324,7 @@ perform_add_pinned_certificate (GckAttributes *search,
                                 GCancellable *cancellable,
                                 GError **error)
 {
-	GckAttributes *attrs;
+	GckBuilder builder = GCK_BUILDER_INIT;
 	gboolean ret = FALSE;
 	GError *lerr = NULL;
 	GckObject *object;
@@ -361,8 +356,7 @@ perform_add_pinned_certificate (GckAttributes *search,
 		return TRUE;
 	}
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_all (attrs, search);
+	gck_builder_add_all (&builder, search);
 
 	/* TODO: Add relevant label */
 
@@ -378,7 +372,8 @@ perform_add_pinned_certificate (GckAttributes *search,
 	} else {
 		session = gck_slot_open_session (slot, CKF_RW_SESSION, NULL, error);
 		if (session != NULL) {
-			object = gck_session_create_object (session, attrs, cancellable, error);
+			object = gck_session_create_object (session, gck_builder_end (&builder),
+			                                    cancellable, error);
 			if (object != NULL) {
 				g_object_unref (object);
 				ret = TRUE;
@@ -390,7 +385,7 @@ perform_add_pinned_certificate (GckAttributes *search,
 		g_object_unref (slot);
 	}
 
-	gck_attributes_unref (attrs);
+	gck_builder_clear (&builder);
 
 	/* Our own local error pointer */
 	g_clear_error (&lerr);
@@ -531,15 +526,13 @@ static GckAttributes *
 prepare_remove_pinned_certificate (GcrCertificate *certificate, const gchar *purpose,
                                    const gchar *peer)
 {
-	GckAttributes *attrs;
+	GckBuilder builder = GCK_BUILDER_INIT;
 
-	attrs = prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE);
-	g_return_val_if_fail (attrs, NULL);
+	prepare_trust_attrs (certificate, CKT_X_PINNED_CERTIFICATE, &builder);
+	gck_builder_add_string (&builder, CKA_X_PURPOSE, purpose);
+	gck_builder_add_string (&builder, CKA_X_PEER, peer);
 
-	gck_attributes_add_string (attrs, CKA_X_PURPOSE, purpose);
-	gck_attributes_add_string (attrs, CKA_X_PEER, peer);
-
-	return attrs;
+	return gck_attributes_ref_sink (gck_builder_end (&builder));
 }
 
 static gboolean
@@ -715,14 +708,12 @@ gcr_trust_remove_pinned_certificate_finish (GAsyncResult *result, GError **error
 static GckAttributes *
 prepare_is_certificate_anchored (GcrCertificate *certificate, const gchar *purpose)
 {
-	GckAttributes *attrs;
+	GckBuilder builder = GCK_BUILDER_INIT;
 
-	attrs = prepare_trust_attrs (certificate, CKT_X_ANCHORED_CERTIFICATE);
-	g_return_val_if_fail (attrs, NULL);
+	prepare_trust_attrs (certificate, CKT_X_ANCHORED_CERTIFICATE, &builder);
+	gck_builder_add_string (&builder, CKA_X_PURPOSE, purpose);
 
-	gck_attributes_add_string (attrs, CKA_X_PURPOSE, purpose);
-
-	return attrs;
+	return gck_attributes_ref_sink (gck_builder_end (&builder));
 }
 
 static gboolean
